//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2013  Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename       : $RCSfile: N_TOP_ParNode.h,v $
//
// Purpose        :
//
// Special Notes  :
//
// Creator        : Rob Hoekstra
//
// Creation Date  : 7/17/01
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.8.6.2 $
//
// Revision Date  : $Date: 2013/10/03 17:23:51 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef  _N_TOP_PARNODE_H
#define  _N_TOP_PARNODE_H

// ---------- Standard Includes ----------
#include <string>
#include <set>
#include <iosfwd>

// ----------   Xyce Includes   ----------
#include <N_TOP_Node.h>

// ----------   Fwd Declares   -----------

// ----------   Defines  -----------------

#define PNP_CAST(x) (reinterpret_cast<N_TOP_ParNode*>(x))

//-----------------------------------------------------------------------------
// Class         : N_TOP_ParNode
// Purpose       :
// Special Notes :
// Creator       : Rob Hoekstra, SNL, Parallel Computational Sciences
// Creation Date : 7/17/01
//-----------------------------------------------------------------------------
class N_TOP_ParNode : public N_TOP_Node
{

public:

  // Constructor
  N_TOP_ParNode( const NodeID & nid = NodeID("", -1),
                 const bool & owned = true,
                 const int & proc = 0)
  : N_TOP_Node(nid, owned),
    proc_(proc)
  {}

  // Constructor
  N_TOP_ParNode(const N_TOP_Node & node, const int & proc = 0)
  : N_TOP_Node(node),
    proc_(proc)
  {}

  // Destructor
  ~N_TOP_ParNode() { }

  N_TOP_Node * clone() const { return new N_TOP_ParNode(* this); }

  N_TOP_Node * expr() const { return new N_TOP_ParNode(); }

  // Simple copy construction, autogenerated by compiler.
  bool operator==(const N_TOP_ParNode & right) const
  { return (nodeID_.first == right.nodeID_.first); }

  // Non-equality operator.
  bool operator!=(const N_TOP_ParNode & right) const
  { return (nodeID_.first != right.nodeID_.first); }

  int & proc() { return proc_; }

  set < int > & ghosts() { return ghosts_; }

  // Packable functionality.
  Packable * instance() const { return new N_TOP_ParNode(); }

  // Counts number of bytes needed to pack object.
  int packedByteCount() const;

  // Packs NodeBlock into char buffer using MPI_PACK.
  void pack(char * buf, int bsize, int & pos, N_PDS_Comm * comm) const;
  void pack(char * buf, int bsize, int & pos, N_PDS_Comm & comm) const
  { pack( buf, bsize, pos, &comm ); }
  // Unpacks NodeBlock from char buffer using MPI_UNPACK.
  void unpack(char * buf, int bsize, int & pos, N_PDS_Comm * comm);
  void unpack(char * buf, int bsize, int & pos, N_PDS_Comm & comm)
  { unpack( buf, bsize, pos, &comm ); }

protected:

  int proc_;

  set<int> ghosts_;

  ostream & put(ostream & os) const;
  friend ostream & operator<<(ostream & os, const N_TOP_ParNode & pn);

};

//-----------------------------------------------------------------------------
// Function      : N_TOP_ParNode::packedByteCount
// Purpose       : Constructor
// Scope         : public
// Special Notes :
// Creator       : Rob Hoekstra, SNL, Parallel Computational Sciences
// Creation Date : 5/18/01
//-----------------------------------------------------------------------------
inline int N_TOP_ParNode::packedByteCount() const
{
  int count = N_TOP_Node::packedByteCount();

  count += sizeof(int); //proc

  count += sizeof(int); //ghosts_ size
  count += ghosts_.size() * sizeof(int); //ghosts_

  return count;
}

#endif
